require 'bigdecimal'

module Bliss
  module Client
    class Style
      include Virtus.model

      attribute :id, Integer
      attribute :number, String
      attribute :name, String
      attribute :treatment, String
      attribute :form, String
      attribute :quality, String
      attribute :category, String
      attribute :collection_id, Integer
      attribute :program_id, Integer
      attribute :prices, Hash

      attribute :articles, Array[Article]

      # @return [BigDecimal] the current purchase (aka wholesale) price, i.e.
      #   the price the stores pay to buy this style from NILE
      #
      # @param [Symbol] currency either :eur or :chf, can also be strings
      #   e.g. 'CHF'
      def purchase_price(currency)
        BigDecimal.new prices_for_currency(currency).
          fetch('current_purchase_price')
      end

      # @return [BigDecimal] the current sales (aka retail) price, i.e. the
      #   price the customers pay to buy this style from a store/webshop
      #
      # @param [Symbol] currency either :eur or :chf, can also be strings
      #   e.g. 'CHF'
      def sales_price(currency)
        BigDecimal.new prices_for_currency(currency).
          fetch('current_sales_price')
      end

      # @return [BigDecimal] the first unreduced price stores paid to purchase
      #   this style from NILE
      #
      # @param [Symbol] currency either :eur or :chf, can also be strings
      #   e.g. 'CHF'
      def original_purchase_price(currency)
        BigDecimal.new prices_for_currency(currency).
          fetch('first_purchase_price')
      end

      # @return [BigDecimal] the first unreduced price customers paid to purchase
      #   this style from a store/webshop
      #
      # @param [Symbol] currency either :eur or :chf, can also be strings
      #   e.g. 'CHF'
      def original_sales_price(currency)
        BigDecimal.new prices_for_currency(currency).
          fetch('first_sales_price')
      end

      # @return [Boolean] whether this style is sold for a reduced price for
      #   the currency
      #
      # @param [Symbol] currency either :eur or :chf, can also be strings
      #   e.g. 'CHF'
      def discounted?(currency)
        prices_for_currency(currency).fetch('current_price_rank').to_i > 1
      end

      private

      def prices_for_currency(currency)
        prices.fetch(currency.to_s.upcase)
      end
    end
  end
end
